C ----------------------------------------------------------------------
      PROGRAM POCONBE
C
C  PROGRAM 1  
C
C 
C  THIS PROGRAM SOLVES TWO DIMENSIONAL (PO)TENTIAL PROBLEMS
C  USING (CON)STANT (B)OUNDARY (E)LEMENTS
C 
C 
      CHARACTER*10 FILEIN,FILEOUT    
C
      DIMENSION X(101),Y(101),XM(100),YM(100),FI(100),DFI(100)
      DIMENSION KODE(100),CX(20),CY(20),SOL(20)
C
      COMMON/MATG/ G(100,100)
      COMMON/MATH/ H(100,100)
      COMMON      N,L,INP,IPR
C
C     SET MAXIMUN DIMENSION OF THE SYSTEM OF EQUATIONS (NX)
C     (THIS NUMBER MUST BE EQUAL OR SMALLER THAN THE DIMENSION OF XM, ETC...)
C     
      NX=100
C
C  ASSIGN NUMBERS FOR INPUT AND OUTPUT FILES
C
      INP=5
      IPR=6      
C
C  READ NAMES AND OPEN  FILES FOR INPUT AND OUTPUT    
C
      WRITE(*,' (A) ') ' NAME OF INPUT FILE (MAX. 10 CHART.)'
      READ(*,'  (A) ')FILEIN
      OPEN(INP,FILE=FILEIN,STATUS='OLD')
      WRITE(*,' (A) ') ' NAME OF OUTPUT FILE (MAX. 10 CHART.)'
      READ(*,' (A) ')FILEOUT
      OPEN(IPR,FILE=FILEOUT,STATUS='NEW')
C
C  READ DATA
C 
      CALL INPUTPC(CX,CY,X,Y,KODE,FI) 
C 
C  COMPUTE H AND G MATRICES AND FORM SYSTEM (A X = F)
C 
      CALL GHMATPC(X,Y,XM,YM,G,H,FI,DFI,KODE,NX) 
C 
C  SOLVE SYSTEM OF EQUATIONS
C 
      CALL SLNPD(G,DFI,D,N,NX)
C 
C  COMPUTE  POTENTIAL VALUES AT INTERNAL POINTS
C 
      CALL INTERPC(FI,DFI,KODE,CX,CY,X,Y,SOL) 
C 
C  PRINT RESULTS AT BOUNDARY NODES AND INTERNAL POINTS 
C 
      CALL OUTPTPC(XM,YM,FI,DFI,CX,CY,SOL)
C
C   CLOSE INPUT AND OUTPUT FILES
C
      CLOSE (INP)
      CLOSE (IPR)
      STOP 
      END 
C-----------------------------------------------------------------------
      SUBROUTINE INPUTPC(CX,CY,X,Y,KODE,FI)
C
C  PROGRAM 2  
C
      CHARACTER*80 TITLE
      COMMON N,L,INP,IPR
      DIMENSION CX(20),CY(20),X(101),Y(101),KODE(100),FI(100)
C 
C  N= NUMBER OF BOUNDARY NODES (=NUMBER OF ELEMENTS) 
C  L= NUMBER OF INTERNAL POINTS WHERE THE FUNCTION IS CALCULATED
C 
      WRITE(IPR,100)
  100 FORMAT(' ',79('*'))
C 
C  READ JOB TITLE 
C 
      READ(INP,'(A)') TITLE
      WRITE(IPR,'(A)') TITLE 
C 
C  READ NUMBER OF BOUNDARY ELEMENTS AND INTERNAL POINTS
C 
      READ(INP,*)N,L
      WRITE(IPR,300)N,L 
  300 FORMAT(//' DATA'//2X,'NUMBER OF BOUNDARY ELEMENTS =',I3/2X,'NUMBER 
     1 OF INTERNAL POINTS WHERE THE FUNCTION IS CALCULATED =',I3) 
C
C 
C  READ COORDINATES OF EXTREME POINTS OF THE BOUNDARY ELEMENTS
C  IN ARRAYS X AND Y 
C 
      WRITE(IPR,500)
  500 FORMAT(//2X,'COORDINATES OF THE EXTREME POINTS OF THE BOUNDARY ELE
     1MENTS',//1X,'POINT',7X,'X',15X,'Y')
      READ(INP,*) (X(I),Y(I),I=1,N) 
      DO 10 I=1,N 
   10 WRITE(IPR,700)I,X(I),Y(I) 
  700 FORMAT(2X,I3,2(2X,E14.5)) 
C 
C  READ BOUNDARY CONDITIONS IN FI(I) VECTOR, IF KODE(I)=0 THE FI(I) 
C  VALUE IS A KNOWN POTENTIAL;IF KODE(I)=1 THE FI(I) VALUE IS A  
C  KNOWN POTENTIAL DERIVATIVE (FLUX).
C 
      WRITE(IPR,800)
  800 FORMAT(//2X,'BOUNDARY CONDITIONS'//2X,'NODE',6X,'CODE',7X,'PRESCRI
     1BED VALUE') 
      DO 20 I=1,N 
      READ(INP,*) KODE(I),FI(I) 
   20 WRITE(IPR,950)I,KODE(I),FI(I) 
  950 FORMAT(2X,I3,9X,I1,8X,E14.5)
C 
C  READ COORDINATES OF THE INTERNAL POINTS 
C
      IF(L.EQ.0) GO TO 30 
      READ(INP,*) (CX(I),CY(I),I=1,L) 
   30 RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE GHMATPC(X,Y,XM,YM,G,H,FI,DFI,KODE,NX) 
C
C  PROGRAM 3 
C
C  THIS SUBROUTINE COMPUTES THE G AND H MATRICES
C  AND FORMS THE SYSTEM OF EQUATIONS A X = F
C
      COMMON N,L,INP,IPR
      DIMENSION X(101),Y(101),XM(100),YM(100),FI(100),KODE(100) 
      DIMENSION DFI(100),G(NX,NX),H(NX,NX)
C 
C  COMPUTE THE NODAL COORDINATES AND STORE IN ARRAYS XM AND YM 
C 
      X(N+1)=X(1) 
      Y(N+1)=Y(1) 
      DO 10 I=1,N 
      XM(I)=(X(I)+X(I+1))/2 
   10 YM(I)=(Y(I)+Y(I+1))/2 
C 
C  COMPUTE THE COEFFICIENTS OF G AND H MATRICES 
C 
      DO 30 I=1,N 
      DO 30 J=1,N 
      KK=J+1
      IF(I-J)20,25,20 
   20 CALL EXTINPC(XM(I),YM(I),X(J),Y(J),X(KK),Y(KK),H(I,J),G(I,J))
      GO TO 30
   25 CALL LOCINPC(X(J),Y(J),X(KK),Y(KK),G(I,J)) 
      H(I,J)=3.1415926
   30 CONTINUE
C 
C  REORDER THE COLUMNS OF THE SYSTEM OF EQUATIONS IN ACCORDANCE
C  WITH THE BOUNDARY CONDITIONS AND FORM SYSTEM MATRIX A WHICH
C  IS STORED IN G
C 
      DO 55 J=1,N 
      IF(KODE(J))55,55,40 
   40 DO 50 I=1,N 
      CH=G(I,J) 
      G(I,J)=-H(I,J)
      H(I,J)=-CH
   50 CONTINUE
   55 CONTINUE
C 
C  FORM THE RIGHT HAND SIDE VECTOR F WHICH IS STORED IN DFI
C 
      DO 60 I=1,N 
      DFI(I)=0. 
      DO 60 J=1,N 
      DFI(I)=DFI(I)+H(I,J)*FI(J)
   60 CONTINUE
      RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE EXTINPC(XP,YP,X1,Y1,X2,Y2,H,G)
C 
C  PROGRAM 4 
C 
C  THIS SUBROUTINE COMPUTES THE OFF DIAGONAL COEFFICIENTS OF
C  H AND G MATRICES USING GAUSS QUADRATURE
C 
C  RA=DISTANCE FROM THE COLLOCATION POINT TO THE
C  GAUSS INTEGRATION POINTS ON THE BOUNDARY ELEMENT
C  DIST=DISTANCE FROM THE COLLOCATION POINT TO THE LINE
C  TANGENT TO THE ELEMENT.
C 
      DIMENSION XCO(4),YCO(4),GI(4),OME(4)
      DATA GI/0.86113631,-0.86113631,0.33998104,-0.33998104/
      DATA OME/0.34785485,0.34785485,0.65214515,0.65214515/
C
      AX=(X2-X1)/2.
      BX=(X2+X1)/2.
      AY=(Y2-Y1)/2.
      BY=(Y2+Y1)/2.
C
C  COMPUTE THE DISTANCE FROM THE POINT TO THE LINE OF THE ELEMENT
C
      IF(AX)10,20,10
   10 TA=AY/AX
      DIST=ABS((TA*XP-YP+Y1-TA*X1)/SQRT(TA**2+1.)) 
      GO TO 30
   20 DIST=ABS(XP-X1)
C
C  DETERMINE THE DIRECTION OF THE OUTWARD NORMAL
C 
   30 SIG=(X1-XP)*(Y2-YP)-(X2-XP)*(Y1-YP) 
      IF(SIG)31,32,32 
   31 DIST=-DIST
   32 G=0.
      H=0.
C
C COMPUTE G AND H COEFFITIENTS
C
      DO 40 I=1,4 
      XCO(I)=AX*GI(I)+BX
      YCO(I)=AY*GI(I)+BY
      RA=SQRT((XP-XCO(I))**2+(YP-YCO(I))**2)
      G=G+ALOG(1/RA)*OME(I)*SQRT(AX**2+AY**2) 
   40 H=H-(DIST*OME(I)*SQRT(AX**2+AY**2)/RA**2) 
      RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE LOCINPC(X1,Y1,X2,Y2,G)
C
C  PROGRAM 5
C 
C  THIS SUBROUTINE COMPUTES THE VALUES OF THE DIAGONAL
C  COEFFITIENTS OF THE G MATRIX 
C 
      AX=(X2-X1)/2.
      AY=(Y2-Y1)/2.
      SR=SQRT(AX**2+AY**2)
      G=2*SR*(1.-ALOG(SR)) 
      RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE SLNPD(A,B,D,N,NX)
C
C  PROGRAM 6
C 
C SOLUTION OF LINEAR SYSTEMS OF EQUATIONS 
C BY THE GAUSS ELIMINATION METHOD PROVIDING 
C FOR INTERCHANGING ROWS WHEN ENCOUNTERING A
C ZERO DIAGONAL COEFICIENT
C 
C A : SYSTEM MATRIX 
C B : ORIGINALLY IT CONTAINS THE INDEPENDENT
C     COEFFICIENTS. AFTER SOLUTION IT CONTAINS
C     THE VALUES OF THE SYSTEM UNKNOWNS.
C 
C N : ACTUAL NUMBER OF UNKNOWNS 
C NX: ROW AND COLUMN DIMENSION OF A 
C 
      DIMENSION B(NX),A(NX,NX)
C
      TOL=1.E-6
C
      N1=N-1
      DO 100 K=1,N1 
      K1=K+1
      C=A(K,K)
      IF(ABS(C)-TOL)1,1,3
    1 DO 7 J=K1,N 
C 
C TRY TO INTERCHANGE ROWS TO GET NON ZERO DIAGONAL COEFFICIENT
C 
      IF(ABS((A(J,K)))-TOL)7,7,5 
    5 DO 6 L=K,N
      C=A(K,L)
      A(K,L)=A(J,L) 
    6 A(J,L)=C
      C=B(K)
      B(K)=B(J) 
      B(J)=C
      C=A(K,K)
      GO TO 3 
    7 CONTINUE
      GO TO 8
C 
C DIVIDE ROW BY DIAGONAL COEFFICIENT
C 
    3 C=A(K,K)
      DO 4 J=K1,N 
    4 A(K,J)=A(K,J)/C 
      B(K)=B(K)/C 
C 
C ELIMINATE UNKNOWN X(K) FROM ROW I 
C 
      DO 10 I=K1,N
      C=A(I,K)
      DO 9 J=K1,N 
    9 A(I,J)=A(I,J)-C*A(K,J)
   10 B(I)=B(I)-C*B(K)
  100 CONTINUE
C 
C COMPUTE LAST UNKNOWN
C 
      IF(ABS((A(N,N)))-TOL)8,8,101 
  101 B(N)=B(N)/A(N,N)
C 
C APPLY BACKSUBSTITUTION PROCESS TO COMPUTE REMAINING UNKNOWNS
C 
      DO 200 L=1,N1 
      K=N-L 
      K1=K+1
      DO 200 J=K1,N 
  200 B(K)=B(K)-A(K,J)*B(J) 
C 
C COMPUTE VALUE OF DETERMINANT
C 
      D=1.
      DO 250 I=1,N
  250 D=D*A(I,I)
      GO TO 300
    8 WRITE(IPR,2) K
    2 FORMAT(' **** SINGULARITY IN ROW',I5) 
      D=0.
  300 RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE INTERPC(FI,DFI,KODE,CX,CY,X,Y,SOL) 
C
C  PROGRAM 7
C
C  THIS SUBROUTINE COMPUTES THE VALUES OF POTENTIAL AT INTERNAL POINTS
C
      COMMON N,L,INP,IPR
      DIMENSION FI(100),DFI(100),KODE(100),CX(20),CY(20),X(101),Y(101)
      DIMENSION SOL(20) 
C 
C  REARRANGE THE FI AND DFI ARRAYS TO STORE ALL THE VALUES OF THE
C  POTENTIAL IN FI AND ALL THE VALUES OF THE DERIVATIVE IN DFI
C 
      DO 20 I=1,N 
      IF(KODE(I)) 20,20,10
   10 CH=FI(I)
      FI(I)=DFI(I)
      DFI(I)=CH 
   20 CONTINUE
C 
C  COMPUTE THE VALUES OF POTENTIAL AT INTERNAL POINTS 
C
      IF(L.EQ.0) GO TO 50  
      DO 40 K=1,L 
      SOL(K)=0. 
      DO 30 J=1,N 
      KK=J+1
      CALL EXTINPC(CX(K),CY(K),X(J),Y(J),X(KK),Y(KK),A,B)
   30 SOL(K)=SOL(K)+DFI(J)*B-FI(J)*A
   40 SOL(K)=SOL(K)/(2*3.1415926) 
   50 RETURN
      END 
C-----------------------------------------------------------------------
      SUBROUTINE OUTPTPC(XM,YM,FI,DFI,CX,CY,SOL)
C
C  PROGRAM 8 
C
C  THIS SUBROUTINE PRINTS THE VALUES OF THE POTENTIAL AND ITS NORMAL 
C  DERIVATIVE AT BOUNDARY NODES. IT ALSO PRINTS THE VALUES OF THE
C  POTENTIAL AT INTERNAL POINTS
C

      COMMON N,L,INP,IPR
      DIMENSION XM(100),YM(100),FI(100),DFI(100),CX(20),CY(20),SOL(20) 
C
      WRITE(IPR,100)
  100 FORMAT(' ',79('*')//1X,'RESULTS'//2X,'BOUNDARY NODES'//8X,'X',15
     1X,'Y',13X,'POTENTIAL',3X,'POTENTIAL DERIVATIVE'/)
      DO 10 I=1,N 
   10 WRITE(IPR,200) XM(I),YM(I),FI(I),DFI(I) 
  200 FORMAT(4(2X,E14.5))
C
      IF(L.EQ.0) GO TO 30
      WRITE(IPR,300)
  300 FORMAT(//,2X,'INTERNAL POINTS',//8X,'X',15X,'Y',13X,'POTENTIAL',/
     1) 
      DO 20 K=1,L 
   20 WRITE(IPR,400)CX(K),CY(K),SOL(K)
  400 FORMAT(3(2X,E14.5)) 
   30 WRITE(IPR,500)
  500 FORMAT(' ',79('*'))
      RETURN
      END 
